# This class sets up the master. For more information, see the [README.md](./README.md)
#
# @param ca_host [String] The hostname of the node acting as a certificate authority.
# @param ca_port [Integer] The port the CA service is listening on.
# @param console_host [String] The hostname of the node acting as the PE Console.
# @param enable_ca_proxy [Boolean] Enable the internal PE CA proxy that will forward agent CA requests to the @ca_host
# @param dashboard_port [Integer] The port the PE console is listening on.
# @param puppetdb_host [String] The hostname of the PuppetDB host.
# @param puppetdb_port [Integer] The port the PuppetDB host is listening on.
# @param console_server_certname [String] The name of the console's SSL certificate.
# @param certname [String] The name of the the master SSL certificate.
# @param classifier_host [String] The hostname of the PE Classifier.
# @param classifier_port [Integer] The port the PE Classifter is listening on.
# @param classifier_url_prefix [String] What to prefix to URLs used with the classifier.
# @param console_client_certname [String]  The name of the certificate to use when connecting to the PE Console.
# @param classifier_client_certname [String] The name of the certificate to use when connecting to the PE Classifier.
# @param dns_alt_names [String] *Deprecated* Alternate DNS names for the master to accept/use.
# @param localcacert [String] Certificate of the CA to use when verifying certificates.
# @param java_args [Hash] Key-value pairs describing the java arguments to be passed when starting the master.
# @param metrics_enabled [Boolean] Flag to enable the master metrics logging system.
# @param metrics_server_id [String] Unique identifier for this server in the metrics backend.
# @param metrics_jmx_enabled [Boolean] Flag to enable JMX on metrics.
# @param metrics_graphite_enabled [Boolean] Flag to enable logging metrics to Graphite.
# @param metrics_graphite_host [String] The name of the Graphite server to log metrics to.
# @param metrics_graphite_port [Integer] the port the Graphite server is listening on.
# @param metrics_graphite_update_interval_seconds [Integer] How often to write metrics to the Graphite server.
# @param profiler_enabled [Boolean] Flag to enable the profiler.
# @param manage_symlinks [Boolean] Flag to enable creation of convenience links
class puppet_enterprise::profile::master(
  $ca_host                                  = $puppet_enterprise::certificate_authority_host,
  $ca_port                                  = $puppet_enterprise::certificate_authority_port,
  $console_host                             = $puppet_enterprise::console_host,
  $enable_ca_proxy                          = true,
  $dashboard_port                           = $puppet_enterprise::params::dashboard_ssl_listen_port,
  $puppetdb_host                            = $puppet_enterprise::puppetdb_host,
  $puppetdb_port                            = $puppet_enterprise::puppetdb_port,
  $console_server_certname                  = $puppet_enterprise::console_host,
  $certname                                 = $::clientcert,
  $classifier_host                          = $puppet_enterprise::console_host,
  $classifier_port                          = $puppet_enterprise::api_port,
  $classifier_url_prefix                    = $puppet_enterprise::classifier_url_prefix,
  $console_client_certname                  = $puppet_enterprise::params::console_client_certname,
  $classifier_client_certname               = $puppet_enterprise::params::classifier_client_certname,
  $dns_alt_names                            = undef,
  $localcacert                              = $puppet_enterprise::params::localcacert,
  $java_args                                = $puppet_enterprise::params::puppetserver_java_args,
  $metrics_enabled                          = true,
  $metrics_server_id                        = $::hostname,
  $metrics_jmx_enabled                      = true,
  $metrics_graphite_enabled                 = false,
  $metrics_graphite_host                    = "graphite",
  $metrics_graphite_port                    = 2003,
  $metrics_graphite_update_interval_seconds = 5,
  $profiler_enabled                         = true,
  $enable_future_parser                     = false,
  $manage_symlinks                          = $puppet_enterprise::manage_symlinks,
) inherits puppet_enterprise {
  pe_validate_bool($enable_future_parser)
  pe_validate_bool($manage_symlinks)

  if ($dns_alt_names != undef) {
    warning('Setting $puppet_enterprise::profile::master::dns_alt_names is no longer necessary; please remove any dns_alt_names from the node classifier.')
  }
  pe_validate_bool($enable_ca_proxy)

  if ($::serverversion and $::serverversion != $::puppetversion) {
    fail("This compile master has a Puppet version of ${::puppetversion} while the master of masters has version ${::serverversion}. Please ensure that the PE version is consistent across all Puppet masters.")
  }

  $confdir = '/etc/puppetlabs/puppet'

  Pe_ini_setting {
    ensure  => present,
    path    => "${confdir}/puppet.conf",
    section => 'master',
  }

  @@puppet_enterprise::certs::rbac_whitelist_entry { "export: ${certname}-for-rbac":
    certname => $certname,
  }

  @@puppet_enterprise::certs::dashboard_whitelist_entry { "export: ${certname}-for-dashboard":
    certname => $certname,
  }

  if $enable_ca_proxy {
    # A reverse proxy between the master and the ca will be configured,
    # allowing agents to have their ca traffic silently forwarded
    $ca_namespace   = 'puppetlabs.enterprise.services.reverse-proxy.reverse-proxy-ca-service'
    $ca_service     = 'reverse-proxy-ca-service'
    $ca_conf_ensure = file
  }
  else {
    # We disable the certificate-authority-service on masters.
    # Relevant resources are collected and overridden in the ca profile
    $ca_namespace   = 'puppetlabs.services.ca.certificate-authority-disabled-service'
    $ca_service     = 'certificate-authority-disabled-service'
    $ca_conf_ensure = absent
  }

  puppet_enterprise::trapperkeeper::bootstrap_cfg { 'certificate-authority-service' :
    container => 'puppetserver',
    namespace => $ca_namespace,
    service   => $ca_service,
    notify    => Service['pe-puppetserver'],
  }

  # Uses
  #   $ca_host
  #   $ca_port
  #   $certname
  #   $localcacert
  file { '/etc/puppetlabs/puppetserver/conf.d/ca.conf':
    ensure  => $ca_conf_ensure,
    content => template('puppet_enterprise/master/puppetserver/ca-proxy.conf.erb'),
    require => Package['pe-puppetserver'],
    notify  => Service['pe-puppetserver'],
  }

  # 'Updating puppet.conf node terminus'
  if $classifier_host {
    class { 'puppet_enterprise::profile::master::classifier' :
      classifier_host       => $classifier_host,
      classifier_port       => $classifier_port,
      classifier_url_prefix => $classifier_url_prefix,
    }
  }

  # Copy the crl from the ca is we're a secondary master
  # and if we're the ca just move it to the hostcrl as the canonical location
  file { $puppet_enterprise::params::hostcrl:
    ensure  => file,
    owner   => 'pe-puppet',
    group   => 'pe-puppet',
    mode    => '0644',
    content => file($::settings::cacrl, $::settings::hostcrl, '/dev/null'),
  }

  # PMT access to pe_only content
  pe_ini_setting { 'module_groups' :
    section              => 'main',
    setting              => 'module_groups',
    value                => 'base+pe_only',
  }

  unless $::custom_auth_conf == 'true' {
    class { 'puppet_enterprise::profile::master::auth_conf' :
      console_client_certname    => $console_client_certname,
      classifier_client_certname => $classifier_client_certname,
    }
  }

  class { 'puppet_enterprise::profile::master::console' :
    console_host            => $console_host,
    dashboard_port          => $dashboard_port,
    console_server_certname => $console_server_certname,
  }

  class { 'puppet_enterprise::profile::master::puppetdb' :
    puppetdb_host             => $puppetdb_host,
    puppetdb_port             => $puppetdb_port,
    soft_write_failure        => false,
  }

  if !pe_empty($java_args) {
    pe_validate_hash($java_args)
  }

  class { 'puppet_enterprise::master' :
    certname                                 => $certname,
    static_files                             => {'/packages' => '/opt/puppet/packages/public'},
    localcacert                              => $localcacert,
    java_args                                => $java_args,
    metrics_enabled                          => $metrics_enabled,
    metrics_server_id                        => $metrics_server_id,
    metrics_jmx_enabled                      => $metrics_jmx_enabled,
    metrics_graphite_enabled                 => $metrics_graphite_enabled,
    metrics_graphite_host                    => $metrics_graphite_host,
    metrics_graphite_port                    => $metrics_graphite_port,
    metrics_graphite_update_interval_seconds => $metrics_graphite_update_interval_seconds,
    profiler_enabled                         => $profiler_enabled,
    enable_future_parser                     => $enable_future_parser,
    manage_symlinks                          => $manage_symlinks,
  }

  # PE MODULE DEPLOYMENT
  # Puppet Enterprise supported deployments need to have a base set of modules
  # installed. These modules are not packaged using rpm or deb, but come from
  # tarball sources. Ensure that each required module is retrieved and
  # deployed.

  $sharedir = $::osfamily ? {
    'windows' => "${::common_appdata}/PuppetLabs/puppet/share",
    default   => '/opt/puppet/share/puppet_enterprise',
  }
  $tarball_staging = "${sharedir}/pe_modules"

  file { $sharedir:
    ensure => directory,
    owner  => $puppet_enterprise::params::root_user,
    group  => $puppet_enterprise::params::root_group,
    mode   => '0755',
  }
  file { $tarball_staging:
    ensure  => directory,
    purge   => true,
    recurse => true,
    force   => true,
    source  => [
      "puppet:///${puppet_enterprise::params::module_mountpoint}/",
      $puppet_enterprise::params::module_tarballsrc,
    ],
    require => Pe_anchor['puppet_enterprise:barrier:ca'],
  }

  # Template uses:
  #   - $tarball_staging
  file { "${tarball_staging}/install.sh":
    ensure  => file,
    mode    => '0755',
    content => template("${module_name}/master/install_module_tarballs.sh.erb"),
  }

  exec { 'Extract PE Modules':
    command     => "${tarball_staging}/install.sh",
    subscribe   => File[$tarball_staging],
    refreshonly => true,
  }

}
